 #if !defined(MQZC_INCLUDED)           /* File not yet included?      */
   #define MQZC_INCLUDED               /* Show file now included      */
 /*********************************************************************/
 /*                                                                   */
 /*                  IBM MQSeries for Linux/Intel                     */
 /*                                                                   */
 /*  HEADER NAME:    CMQZC                                            */
 /*                                                                   */
 /*  DESCRIPTION:    Declarations for Installable Services            */
 /*                                                                   */
 /*********************************************************************/
 /*  @START_COPYRIGHT@                                                */
 /*  Licensed Materials - Property of IBM                             */
 /*                                                                   */
 /*  0783881, 5639-I84                                                */
 /*                                                                   */
 /*  (C) Copyright IBM Corporation 1995, 2000.                        */
 /*                                                                   */
 /*  Status: Version 5 Release 2                                      */
 /*  @END_COPYRIGHT@                                                  */
 /*********************************************************************/
 /*                                                                   */
 /*  FUNCTION:       This file declares the functions, structures,    */
 /*                  and named constants for installable services.    */
 /*                                                                   */
 /*  PROCESSOR:      C                                                */
 /*                                                                   */
 /*********************************************************************/

 #if defined(__cplusplus)
   extern "C" {
 #endif

 /*********************************************************************/
 /*  Values Related to MQZED Structure                                */
 /*********************************************************************/

 /* Structure Identifier */
 #define MQZED_STRUC_ID "ZED "

 /* Structure Identifier (array form) */
 #define MQZED_STRUC_ID_ARRAY 'Z','E','D',' '

 /* Structure Version Number */
 #define MQZED_VERSION_1       1L
 #define MQZED_CURRENT_VERSION 1L


 /*********************************************************************/
 /*  Values related to MQZEP function                                 */
 /*********************************************************************/

 /* Function ids common to all services */
 #define MQZID_INIT 0L
 #define MQZID_TERM 1L

 /* Function ids for Authority service */
 #define MQZID_INIT_AUTHORITY         0L
 #define MQZID_TERM_AUTHORITY         1L
 #define MQZID_CHECK_AUTHORITY        2L
 #define MQZID_COPY_ALL_AUTHORITY     3L
 #define MQZID_DELETE_AUTHORITY       4L
 #define MQZID_SET_AUTHORITY          5L
 #define MQZID_GET_AUTHORITY          6L
 #define MQZID_GET_EXPLICIT_AUTHORITY 7L
 #define MQZID_REFRESH_CACHE          8L

 /* Function ids for Name service */
 #define MQZID_INIT_NAME   0L
 #define MQZID_TERM_NAME   1L
 #define MQZID_LOOKUP_NAME 2L
 #define MQZID_INSERT_NAME 3L
 #define MQZID_DELETE_NAME 4L

 /* Function ids for Userid service */
 #define MQZID_INIT_USERID 0L
 #define MQZID_TERM_USERID 1L
 #define MQZID_FIND_USERID 2L


 /*********************************************************************/
 /*  Values related to all services                                   */
 /*********************************************************************/

 /* Initialization Options */
 #define MQZIO_PRIMARY   0L
 #define MQZIO_SECONDARY 1L

 /* Termination Options */
 #define MQZTO_PRIMARY   0L
 #define MQZTO_SECONDARY 1L

 /* Continuation Indicator */
 #define MQZCI_DEFAULT  0L
 #define MQZCI_CONTINUE 0L
 #define MQZCI_STOP     1L


 /*********************************************************************/
 /*  Values related to Authority service                              */
 /*********************************************************************/

 /* Service Interface Version */
 #define MQZAS_VERSION_1 1L
 #define MQZAS_VERSION_2 2L
 #define MQZAS_VERSION_3 3L

 /* Authorizations */
 #define MQZAO_CONNECT                  0x00000001L
 #define MQZAO_BROWSE                   0x00000002L
 #define MQZAO_INPUT                    0x00000004L
 #define MQZAO_OUTPUT                   0x00000008L
 #define MQZAO_INQUIRE                  0x00000010L
 #define MQZAO_SET                      0x00000020L
 #define MQZAO_PASS_IDENTITY_CONTEXT    0x00000040L
 #define MQZAO_PASS_ALL_CONTEXT         0x00000080L
 #define MQZAO_SET_IDENTITY_CONTEXT     0x00000100L
 #define MQZAO_SET_ALL_CONTEXT          0x00000200L
 #define MQZAO_ALTERNATE_USER_AUTHORITY 0x00000400L
 #define MQZAO_ALL_MQI                  0x000007ffL
 #define MQZAO_CREATE                   0x00010000L
 #define MQZAO_DELETE                   0x00020000L
 #define MQZAO_DISPLAY                  0x00040000L
 #define MQZAO_CHANGE                   0x00080000L
 #define MQZAO_CLEAR                    0x00100000L
 #define MQZAO_AUTHORIZE                0x00800000L
 #define MQZAO_ALL_ADMIN                0x009e0000L
 #define MQZAO_ALL                      0x009e07ffL
 #define MQZAO_NONE                     0x00000000L

 /* Entity Types */
 #define MQZAET_PRINCIPAL 0x00000001L
 #define MQZAET_GROUP     0x00000002L


 /*********************************************************************/
 /*  Values related to Name service                                   */
 /*********************************************************************/

 /* Service Interface Version */
 #define MQZNS_VERSION_1 1L


 /*********************************************************************/
 /*  Values related to Userid service                                 */
 /*********************************************************************/

 /* Service Interface Version */
 #define MQZUS_VERSION_1 1L


 /*********************************************************************/
 /*  MQZED Structure -- Entity Data                                   */
 /*********************************************************************/

 typedef struct tagMQZED {
   MQCHAR4   StrucId;          /* Structure identifier */
   MQLONG    Version;          /* Structure version number */
   PMQCHAR   EntityNamePtr;    /* Address of entity name */
   PMQCHAR   EntityDomainPtr;  /* Address of entity domain name */
   MQBYTE40  SecurityId;       /* Security identifier */
 } MQZED;
 typedef MQZED MQPOINTER PMQZED;

 #define MQZED_DEFAULT {MQZED_STRUC_ID_ARRAY},\
                       MQZED_VERSION_1,\
                       NULL,\
                       NULL,\
                       {MQSID_NONE_ARRAY}


 /*********************************************************************/
 /*  MQZEP -- Add Component Entry Point                               */
 /*********************************************************************/

 void MQENTRY MQZEP (
   MQHCONFIG  Hconfig,     /* Configuration handle */
   MQLONG     Function,    /* Function identifier */
   PMQFUNC    EntryPoint,  /* Function entry point */
   PMQLONG    pCompCode,   /* Completion code */
   PMQLONG    pReason);    /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  Function datatypes for Authority services                        */
 /*********************************************************************/

 /*********************************************************************/
 /*  MQZ_INIT_AUTHORITY -- Initialize Authority-Services              */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_INIT_AUTHORITY (
     MQHCONFIG  Hconfig,              /* Configuration handle */
     MQLONG     Options,              /* Initialization options */
     PMQCHAR    pQMgrName,            /* Queue manager name */
     MQLONG     ComponentDataLength,  /* Length of component data */
     PMQBYTE    pComponentData,       /* Component data */
     PMQLONG    pVersion,             /* Version number */
     PMQLONG    pCompCode,            /* Completion code */
     PMQLONG    pReason);             /* Reason code qualifying
                                         CompCode */


 /*********************************************************************/
 /*  MQZ_TERM_AUTHORITY -- Terminate Authority-Services               */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_TERM_AUTHORITY (
     MQHCONFIG  Hconfig,         /* Configuration handle */
     MQLONG     Options,         /* Termination options */
     PMQCHAR    pQMgrName,       /* Queue manager name */
     PMQBYTE    pComponentData,  /* Component data */
     PMQLONG    pCompCode,       /* Completion code */
     PMQLONG    pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_DELETE_AUTHORITY -- Delete Authority                         */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_DELETE_AUTHORITY (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_GET_AUTHORITY -- Get Authority                               */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_GET_AUTHORITY (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pEntityName,     /* Entity name */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     PMQLONG  pAuthority,      /* Authority of entity */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_GET_AUTHORITY_2 -- Get Authority Version 2                   */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_GET_AUTHORITY_2 (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQZED   pEntityData,     /* Entity data */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     PMQLONG  pAuthority,      /* Authority of entity */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_GET_EXPLICIT_AUTHORITY -- Get Explicit Authority             */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_GET_EXPLICIT_AUTHORITY (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pEntityName,     /* Entity name */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     PMQLONG  pAuthority,      /* Authority of entity */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_GET_EXPLICIT_AUTHORITY_2 -- Get Explicit Authority Version 2 */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_GET_EXPLICIT_AUTHORITY_2 (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQZED   pEntityData,     /* Entity data */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     PMQLONG  pAuthority,      /* Authority of entity */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_SET_AUTHORITY -- Set Authority                               */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_SET_AUTHORITY (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pEntityName,     /* Entity name */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     MQLONG   Authority,       /* Authority to be checked */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_SET_AUTHORITY_2 -- Set Authority Version 2                   */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_SET_AUTHORITY_2 (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQZED   pEntityData,     /* Entity data */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     MQLONG   Authority,       /* Authority to be checked */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_COPY_ALL_AUTHORITY -- Copy All Authority                     */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_COPY_ALL_AUTHORITY (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pRefObjectName,  /* Reference object name */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_CHECK_AUTHORITY -- Check Authority                           */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_CHECK_AUTHORITY (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pEntityName,     /* Entity name */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     MQLONG   Authority,       /* Authority to be checked */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_CHECK_AUTHORITY_2 -- Check Authority Version 2               */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_CHECK_AUTHORITY_2 (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQZED   pEntityData,     /* Entity data */
     MQLONG   EntityType,      /* Entity type */
     PMQCHAR  pObjectName,     /* Object name */
     MQLONG   ObjectType,      /* Object type */
     MQLONG   Authority,       /* Authority to be checked */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_REFRESH_CACHE -- Refresh Cache                               */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_REFRESH_CACHE (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  Function datatypes for Name services                             */
 /*********************************************************************/

 /*********************************************************************/
 /*  MQZ_INIT_NAME -- Initialize Name-Services                        */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_INIT_NAME (
     MQHCONFIG  Hconfig,              /* Configuration handle */
     MQLONG     Options,              /* Initialization options */
     PMQCHAR    pQMgrName,            /* Queue manager name */
     MQLONG     ComponentDataLength,  /* Length of component data */
     PMQBYTE    pComponentData,       /* Component data */
     PMQLONG    pVersion,             /* Version number */
     PMQLONG    pCompCode,            /* Completion code */
     PMQLONG    pReason);             /* Reason code qualifying
                                         CompCode */


 /*********************************************************************/
 /*  MQZ_TERM_NAME -- Terminate Name-Services                         */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_TERM_NAME (
     MQHCONFIG  Hconfig,         /* Configuration handle */
     MQLONG     Options,         /* Termination options */
     PMQCHAR    pQMgrName,       /* Queue manager name */
     PMQBYTE    pComponentData,  /* Component data */
     PMQLONG    pCompCode,       /* Completion code */
     PMQLONG    pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_LOOKUP_NAME -- Look-Up Name                                  */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_LOOKUP_NAME (
     PMQCHAR  pQMgrName,          /* Queue manager name */
     PMQCHAR  pQName,             /* Queue name */
     PMQCHAR  pResolvedQMgrName,  /* Resolved queue manager name */
     PMQBYTE  pComponentData,     /* Component data */
     PMQLONG  pContinuation,      /* Continuation indicator set by
                                     component */
     PMQLONG  pCompCode,          /* Completion code */
     PMQLONG  pReason);           /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_INSERT_NAME -- Insert Name                                   */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_INSERT_NAME (
     PMQCHAR  pQMgrName,          /* Queue manager name */
     PMQCHAR  pQName,             /* Queue name */
     PMQCHAR  pResolvedQMgrName,  /* Resolved queue manager name */
     PMQBYTE  pComponentData,     /* Component data */
     PMQLONG  pContinuation,      /* Continuation indicator set by
                                     component */
     PMQLONG  pCompCode,          /* Completion code */
     PMQLONG  pReason);           /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_DELETE_NAME -- Delete Name                                   */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_DELETE_NAME (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pQName,          /* Queue name */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  Function datatypes for Userid services                           */
 /*********************************************************************/

 /*********************************************************************/
 /*  MQZ_INIT_USERID -- Initialize Userid-Services                    */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_INIT_USERID (
     MQHCONFIG  Hconfig,              /* Configuration handle */
     MQLONG     Options,              /* Initialization options */
     PMQCHAR    pQMgrName,            /* Queue manager name */
     MQLONG     ComponentDataLength,  /* Length of component data */
     PMQBYTE    pComponentData,       /* Component data */
     PMQLONG    pVersion,             /* Version number */
     PMQLONG    pCompCode,            /* Completion code */
     PMQLONG    pReason);             /* Reason code qualifying
                                         CompCode */


 /*********************************************************************/
 /*  MQZ_TERM_USERID -- Terminate Userid-Services                     */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_TERM_USERID (
     MQHCONFIG  Hconfig,         /* Configuration handle */
     MQLONG     Options,         /* Termination options */
     PMQCHAR    pQMgrName,       /* Queue manager name */
     PMQBYTE    pComponentData,  /* Component data */
     PMQLONG    pCompCode,       /* Completion code */
     PMQLONG    pReason);        /* Reason code qualifying CompCode */


 /*********************************************************************/
 /*  MQZ_FIND_USERID -- Find Userid                                   */
 /*********************************************************************/

 typedef

   void MQENTRY MQZ_FIND_USERID (
     PMQCHAR  pQMgrName,       /* Queue manager name */
     PMQCHAR  pUserid,         /* User identifier */
     PMQCHAR  pPassword,       /* Password */
     PMQBYTE  pComponentData,  /* Component data */
     PMQLONG  pContinuation,   /* Continuation indicator set by
                                  component */
     PMQLONG  pCompCode,       /* Completion code */
     PMQLONG  pReason);        /* Reason code qualifying CompCode */


 #if defined(__cplusplus)
   }
 #endif

 /*********************************************************************/
 /*  End of CMQZC                                                     */
 /*********************************************************************/
 #endif  /* End of header file */
