#ifndef _IMQCAC_HPP_
#define _IMQCAC_HPP_

//  Library:       MQSeries
//  Component:     IMQB (MQSeries C++ MQI)
//  Part:          IMQCAC.HPP
//
//  Description:   "ImqCache" class declaration
//  <START_COPYRIGHT>
//  Statement:     Licensed Materials - Property of IBM                          
//                                                               
//                 04L1773, 5765-B73
//                 04L1788, 5765-B74
//                 04L1816, 5765-B75
//                 04L1802, 5639-B42
//                 04L1830, 5639-B43
//                 (C) Copyright IBM Corp. 1994, 1999                  
//
//                 U.S. Government Users Restricted Rights -
//                 use, duplication or disclosure restricted by
//                 GSA ADP Schedule Contract with IBM Corp.
//  <END_COPYRIGHT>

extern "C" {
#include <string.h>
}

#include <imqerr.hpp> // ImqError

#define ImqCache ImqCac

class ImqCache : public virtual ImqError {
  char * opszBuffer ;
  size_t ouiDataOffset ;
  size_t ouiBufferLength ;
  size_t ouiMessageLength ;
  ImqBoolean obAutomaticBuffer ;
  char ocPadding1[ 3 ];
  // New methods:
  void setAutomaticBuffer ( const ImqBoolean bAutomatic )
    { obAutomaticBuffer = bAutomatic ; }
public :
  // New methods:
  ImqCache ( );
  ImqCache ( const ImqCache & );
  virtual ~ ImqCache ( );
  void operator = ( const ImqCache & );
  ImqBoolean automaticBuffer ( ) const { return obAutomaticBuffer ; }
  size_t bufferLength ( ) const { return ouiBufferLength ; }
  char * bufferPointer ( ) const { return opszBuffer ; }
  void clearMessage ( );
  size_t dataLength ( ) const ;
  size_t dataOffset ( ) const { return ouiDataOffset ; }
  char * dataPointer ( ) const { return bufferPointer( ) + dataOffset( ); }
  size_t messageLength ( ) const { return ouiMessageLength ; }
  ImqBoolean moreBytes ( const size_t );
  ImqBoolean read ( const size_t, char * & );
  ImqBoolean resizeBuffer ( const size_t );
  ImqBoolean setDataOffset ( const size_t );
  ImqBoolean setMessageLength ( const size_t );
  ImqBoolean useEmptyBuffer ( const char *, const size_t );
  ImqBoolean useFullBuffer ( const char *, const size_t );
  ImqBoolean write ( const size_t, const char * );
} ;

#endif
