#ifndef _IMQDLH_HPP_
#define _IMQDLH_HPP_

//  Library:       MQSeries
//  Component:     IMQI (MQSeries C++ MQI)
//  Part:          IMQDLH.HPP
//
//  Description:   "ImqDeadLetterHeader" class declaration
//  <START_COPYRIGHT>
//  Statement:     Licensed Materials - Property of IBM
//
//                 04L1773, 5765-B73
//                 04L1788, 5765-B74
//                 04L1816, 5765-B75
//                 04L1802, 5639-B42
//                 04L1830, 5639-B43
//                 (C) Copyright IBM Corp. 1994, 1999
//
//                 U.S. Government Users Restricted Rights -
//                 use, duplication or disclosure restricted by
//                 GSA ADP Schedule Contract with IBM Corp.
//  <END_COPYRIGHT>

#include <imqhdr.hpp> // ImqHeader

extern "C" {
typedef struct tagMQDLH1 {
  MQCHAR4 StrucId ;
  MQLONG Version ;
  MQLONG Reason ;
  MQCHAR48 DestQName ;
  MQCHAR48 DestQMgrName ;
  MQLONG Encoding ;
  MQLONG CodedCharSetId ;
  MQCHAR8 Format ;
  MQLONG PutApplType ;
  MQCHAR28 PutApplName ;
  MQCHAR8 PutDate ;
  MQCHAR8 PutTime ;
} MQDLH1 ;
typedef MQDLH1 MQPOINTER PMQDLH1 ;
}

#define ImqDeadLetterHeader ImqDlh

class ImqDeadLetterHeader : public ImqHeader {
  MQLONG olHeaderFlags ;
protected :
  MQDLH1 omqdlh ;
public :
  // Overloaded "ImqItem" methods:
  virtual ImqBoolean copyOut ( ImqMsg & );
  virtual ImqBoolean pasteIn ( ImqMsg & );
  // Overloaded "ImqHeader" methods:
  virtual MQLONG characterSet ( ) const ;
  virtual MQLONG encoding ( ) const ;
  virtual ImqString format ( ) const ;
  virtual MQLONG headerFlags ( ) const ;
  virtual void setCharacterSet ( const MQLONG = MQCCSI_Q_MGR );
  virtual void setEncoding ( const MQLONG = MQENC_NATIVE );
  virtual void setFormat ( const char * = 0 );
  virtual void setHeaderFlags ( const MQLONG = 0 );
  // New methods:
  ImqDeadLetterHeader ( );
  ImqDeadLetterHeader ( const ImqDeadLetterHeader & );
  virtual ~ ImqDeadLetterHeader ( );
  void operator = ( const ImqDeadLetterHeader & );
  MQLONG deadLetterReasonCode ( ) const { return omqdlh.Reason ; }
  ImqString destinationQueueManagerName ( ) const ;
  ImqString destinationQueueName ( ) const ;
  ImqString putApplicationName ( ) const ;
  MQLONG putApplicationType ( ) const { return omqdlh.PutApplType ; }
  ImqString putDate ( ) const ;
  ImqString putTime ( ) const ;
  void setDestinationQueueManagerName ( const char * );
  void setDestinationQueueName ( const char * );
  void setPutApplicationName ( const char * = 0 );
  void setPutApplicationType ( const MQLONG lType = MQAT_NO_CONTEXT )
    { omqdlh.PutApplType = lType ; }
  void setPutDate ( const char * = 0 );
  void setPutTime ( const char * = 0 );
  void setDeadLetterReasonCode ( const MQLONG lReason )
    { omqdlh.Reason = lReason ; }
} ;

#endif
