#ifndef _IMQMSG_HPP_
#define _IMQMSG_HPP_

//  Library:       MQSeries
//  Component:     IMQI (MQSeries C++ MQI)
//  Part:          IMQMSG.HPP
//
//  Description:   "ImqMessage" class declaration
//  <START_COPYRIGHT>
//  Statement:     Licensed Materials - Property of IBM
//
//                 04L1773, 5765-B73
//                 04L1788, 5765-B74
//                 04L1816, 5765-B75
//                 04L1802, 5639-B42
//                 04L1830, 5639-B43
//                 (C) Copyright IBM Corp. 1994, 1999
//
//                 U.S. Government Users Restricted Rights -
//                 use, duplication or disclosure restricted by
//                 GSA ADP Schedule Contract with IBM Corp.
//  <END_COPYRIGHT>

#include <imqbin.hpp> // ImqBinary
#include <imqcac.hpp> // ImqCache
#include <imqmtr.hpp> // ImqMessageTracker
#include <imqstr.hpp> // ImqString

extern "C" {
typedef struct tagMQMD2 {
  MQCHAR4 StrucId ;
  MQLONG Version ;
  MQLONG Report ;
  MQLONG MsgType ;
  MQLONG Expiry ;
  MQLONG Feedback ;
  MQLONG Encoding ;
  MQLONG CodedCharSetId ;
  MQCHAR8 Format ;
  MQLONG Priority ;
  MQLONG Persistence ;
  MQBYTE24 MsgId ;
  MQBYTE24 CorrelId ;
  MQLONG BackoutCount ;
  MQCHAR48 ReplyToQ ;
  MQCHAR48 ReplyToQMgr ;
  MQCHAR12 UserIdentifier ;
  MQBYTE32 AccountingToken ;
  MQCHAR32 ApplIdentityData ;
  MQLONG PutApplType ;
  MQCHAR28 PutApplName ;
  MQCHAR8 PutDate ;
  MQCHAR8 PutTime ;
  MQCHAR4 ApplOriginData ;
  MQBYTE24 GroupId ;
  MQLONG MsgSeqNumber ;
  MQLONG Offset ;
  MQLONG MsgFlags ;
  MQLONG OriginalLength ;
} MQMD2 ;
typedef MQMD2 MQPOINTER PMQMD2 ;
}

#define ImqMessage                        ImqMsg

class ImqItm ;
class ImqQue ;
class ImqMessage : public ImqCache, public ImqMessageTracker {
protected :
  MQMD2 omqmd ;
  size_t ouiTotalMessageLength ;
  friend class ImqQue ;
  static void setVersionSupported ( const MQLONG );
public :
  // New methods:
  ImqMessage ( );
  ImqMessage ( const ImqMessage & );
  virtual ~ ImqMessage ( );
  void operator = ( const ImqMessage & );
  ImqString applicationIdData ( ) const ;
  ImqString applicationOriginData ( ) const ;
  MQLONG backoutCount ( ) const { return omqmd.BackoutCount ; }
  MQLONG characterSet ( ) const { return omqmd.CodedCharSetId ; }
  MQLONG encoding ( ) const { return omqmd.Encoding ; }
  MQLONG expiry ( ) const { return omqmd.Expiry ; }
  ImqString format ( ) const ;
  ImqBoolean formatIs ( const char * ) const ;
  MQLONG messageFlags ( ) const { return omqmd.MsgFlags ; }
  MQLONG messageType ( ) const { return omqmd.MsgType ; }
  MQLONG offset ( ) const { return omqmd.Offset ; }
  MQLONG originalLength ( ) const { return omqmd.OriginalLength ; }
  MQLONG persistence ( ) const { return omqmd.Persistence ; }
  MQLONG priority ( ) const { return omqmd.Priority ; }
  ImqString putApplicationName ( ) const ;
  MQLONG putApplicationType ( ) const { return omqmd.PutApplType ; }
  ImqString putDate ( ) const ;
  ImqString putTime ( ) const ;
  ImqBoolean readItem ( ImqItm & );
  ImqString replyToQueueManagerName ( ) const ;
  ImqString replyToQueueName ( ) const ;
  MQLONG report ( ) const { return omqmd.Report ; }
  MQLONG sequenceNumber ( ) const { return omqmd.MsgSeqNumber ; }
  void setApplicationIdData ( const char * = 0 );
  void setApplicationOriginData ( const char * = 0 );
  void setCharacterSet ( const MQLONG l = MQCCSI_Q_MGR )
    { omqmd.CodedCharSetId = l ; }
  void setEncoding ( const MQLONG l = MQENC_NATIVE ) { omqmd.Encoding = l ; }
  void setExpiry ( const MQLONG l ) { omqmd.Expiry = l ; }
  void setFormat ( const char * = 0 );
  void setMessageFlags ( const MQLONG l ) { omqmd.MsgFlags = l ; }
  void setMessageType ( const MQLONG l ) { omqmd.MsgType = l ; }
  void setOffset ( const MQLONG l ) { omqmd.Offset = l ; }
  void setOriginalLength ( const MQLONG l ) { omqmd.OriginalLength = l ; }
  void setPersistence ( const MQLONG l ) { omqmd.Persistence = l ; }
  void setPriority ( const MQLONG l ) { omqmd.Priority = l ; }
  void setPutApplicationName ( const char * = 0 );
  void setPutApplicationType ( const MQLONG l = MQAT_NO_CONTEXT )
    { omqmd.PutApplType = l ; }
  void setPutDate ( const char * = 0 );
  void setPutTime ( const char * = 0 );
  void setReplyToQueueManagerName ( const char * = 0 );
  void setReplyToQueueName ( const char * = 0 );
  void setReport ( const MQLONG l ) { omqmd.Report = l ; }
  void setSequenceNumber ( const MQLONG l ) { omqmd.MsgSeqNumber = l ; }
  void setUserId ( const char * = 0 );
  size_t totalMessageLength ( ) const { return ouiTotalMessageLength ; }
  ImqString userId ( ) const ;
  ImqBoolean writeItem ( ImqItm & );
} ;

#endif
