/* @(#) samples/c/amqsaxe0.c, lqm, p520, p520-CSD07G  1.5 02/01/08 13:01:22 */
 /********************************************************************/
 /*                                                                  */
 /* Program name: AMQSWLM0                                           */
 /*                                                                  */
 /* Description: Sample ApiExit which traces MQAPI calls             */
 /* <START_COPYRIGHT>                                                */
 /* Licensed Materials - Property of IBM                             */
 /*                                                                  */
 /* 47P8479, 5639-B43                                                */
 /*                                                                  */
 /* (C) Copyright IBM Corporation 1994, 2001 All Rights Reserved     */
 /* US Government Users Restricted Rights - Use, duplication or      */
 /* disclosure restricted by GSA ADP Schedule Contract with IBM Corp. */
 /*                                                                  */
 /* <END_COPYRIGHT>                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*                                                                  */
 /*   AMQSAXE is a sample C ApiExit which traces MQAPI calls         */
 /*                                                                  */
 /*      -- Configure the queue manager                              */
 /*                                                                  */
 /*         On UNIX add a stanza like the following to the           */
 /*         "qm.ini" file                                            */
 /*                                                                  */
 /*         ApiExitLocal:                                            */
 /*           Sequence=100                                           */
 /*           Function=EntryPoint                                    */
 /*           Module=/opt/mqm/bin/amqsaxe.so                         */
 /*           Name=SampleApiExit                                     */
 /*                                                                  */
 /*         On Windows set the equivalent attributes in the registry */
 /*         using the MQ services snapin utility                     */
 /*                                                                  */
 /*      -- Make sure the Module is accessible to MQ                 */
 /*                                                                  */
 /*      -- Restart the Queue Manager to pick up these attributes    */
 /*                                                                  */
 /*      -- In the application process to be traced, describe where  */
 /*         the trace files should be written to.  For example:      */
 /*                                                                  */
 /*         On UNIX, make sure the directory "/var/mqm/trace" exists */
 /*         and export the following environment variable            */
 /*                                                                  */
 /*         "export MQAPI_TRACE_LOGFILE=/var/mqm/trace/ApiTrace"     */
 /*                                                                  */
 /*         On Windows, make sure the directory "C:\temp" exists     */
 /*         and set the following environment variable               */
 /*                                                                  */
 /*         "set MQAPI_TRACE_LOGFILE=C:\temp\ApiTrace"               */
 /*                                                                  */
 /*      -- Optionally in the application process to be traced,      */
 /*         set the options by setting the following environment     */
 /*         variable                                                 */
 /*                                                                  */
 /*         On UNIX                                                  */
 /*                                                                  */
 /*         "export MQAPI_TRACE_OPTIONS=<value>"                     */
 /*                                                                  */
 /*         On Windows                                               */
 /*                                                                  */
 /*         "set MQAPI_TRACE_OPTIONS=<value>"                        */
 /*                                                                  */
 /*         Where <value> is a combination following:                */
 /*                                                                  */
 /*              1    Write context at the start of trace            */
 /*              2    Write context before each MQAPI call           */
 /*              4    Write context in all other cases               */
 /*              8    Write parms   at the start of trace            */
 /*             16    Write parms   before each MQAPI call           */
 /*             32    Write parms   in all other cases               */
 /*             64    Trace Before Dataconv on MQGET                 */
 /*            128    Dump ExitChainAreaPtr when dumping ExitParms   */
 /*                                                                  */
 /*         If MQAPI_TRACE_OPTIONS is not set and the ExitData is    */
 /*         not empty, the options will be set to the numeric value  */
 /*         of ExitData                                              */
 /*                                                                  */
 /*         Otherwise options default to "1"                         */
 /*                                                                  */
 /********************************************************************/

 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
 #include <ctype.h>
 #include <time.h>

 #include <cmqc.h>
 #include <cmqxc.h>

 #if (MQAT_DEFAULT == MQAT_WINDOWS_NT)
 #include <windows.h>
 #include <winbase.h>
 #endif

 #if (MQAT_DEFAULT == MQAT_UNIX) || (MQAT_DEFAULT == MQAT_OS400)
 #include <sys/time.h>
 #endif

 /********************************************************************/
 /* Macros to find the max or min of a pair of numbers               */
 /********************************************************************/
 #ifndef max
   #define max(a,b)        (((a) > (b)) ? (a) : (b))
 #endif

 #ifndef min
   #define min(a,b)        (((a) < (b)) ? (a) : (b))
 #endif

/*********************************************************************/
/* Definitions of data structures                                    */
/*********************************************************************/

typedef struct myExitUserArea
{
  FILE        * fp;
  MQLONG        Options;

#if (MQAT_DEFAULT == MQAT_WINDOWS_NT)
  LARGE_INTEGER PerformanceCounter;
  double        Frequency;
#endif

} MYEXITUSERAREA;

#define OPTIONS_DUMP_CONTEXT_AT_START    0x0001
#define OPTIONS_DUMP_CONTEXT_BEFORE      0x0002
#define OPTIONS_DUMP_CONTEXT_ALWAYS      0x0004
#define OPTIONS_DUMP_PARMS_AT_START      0x0008
#define OPTIONS_DUMP_PARMS_BEFORE        0x0010
#define OPTIONS_DUMP_PARMS_ALWAYS        0x0020
#define OPTIONS_DUMP_DATACONV            0x0040
#define OPTIONS_DUMP_EXITCHAINAREA       0x0080
#define OPTIONS_DEFAULT                  ( OPTIONS_DUMP_CONTEXT_AT_START  \
                                         )
#define TITLE_FORMAT        " %-25.25s %20.20s %26.26s\n"

/*********************************************************************/
/*                                                                   */
/* Standard MQ Entrypoint                                            */
/*                                                                   */
/*********************************************************************/

void MQStart(){;}

/*********************************************************************/
/*                                                                   */
/* Convert an environment value into a string                        */
/*                                                                   */
/*********************************************************************/

char * strEnvironment ( MQLONG Environment, char *Buffer )
{
  char *p = NULL;

  switch (Environment)
  {
    case MQXE_OTHER:           p = "MQXE_OTHER";           break;
    case MQXE_MCA:             p = "MQXE_MCA";             break;
    case MQXE_MCA_SVRCONN:     p = "MQXE_MCA_SVRCONN";     break;
    case MQXE_COMMAND_SERVER:  p = "MQXE_COMMAND_SERVER";  break;
    case MQXE_MQSC:            p = "MQXE_MQSC";            break;
    default:
      sprintf(Buffer, "%d", Environment);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an appl type value into a string                          */
/*                                                                   */
/*********************************************************************/

char * strApplType ( MQLONG ApplType, char *Buffer )
{
  char *p = NULL;

  switch (ApplType)
  {
    case MQAT_UNKNOWN:         p = "MQAT_UNKNOWN";         break;
    case MQAT_NO_CONTEXT:      p = "MQAT_NO_CONTEXT";      break;
    case MQAT_CICS:            p = "MQAT_CICS";            break;
    case MQAT_MVS:             p = "MQAT_MVS";             break;
    case MQAT_IMS:             p = "MQAT_IMS";             break;
    case MQAT_OS2:             p = "MQAT_OS2";             break;
    case MQAT_DOS:             p = "MQAT_DOS";             break;
    case MQAT_UNIX:            p = "MQAT_UNIX";            break;
    case MQAT_QMGR:            p = "MQAT_QMGR";            break;
    case MQAT_OS400:           p = "MQAT_OS400";           break;
    case MQAT_WINDOWS:         p = "MQAT_WINDOWS";         break;
    case MQAT_CICS_VSE:        p = "MQAT_CICS_VSE";        break;
    case MQAT_WINDOWS_NT:      p = "MQAT_WINDOWS_NT";      break;
    case MQAT_VMS:             p = "MQAT_VMS";             break;
    case MQAT_GUARDIAN:        p = "MQAT_GUARDIAN";        break;
    case MQAT_VOS:             p = "MQAT_VOS";             break;
    case MQAT_IMS_BRIDGE:      p = "MQAT_IMS_BRIDGE";      break;
    case MQAT_XCF:             p = "MQAT_XCF";             break;
    case MQAT_CICS_BRIDGE:     p = "MQAT_CICS_BRIDGE";     break;
    case MQAT_NOTES_AGENT:     p = "MQAT_NOTES_AGENT";     break;
    case MQAT_BROKER:          p = "MQAT_BROKER";          break;
    case MQAT_JAVA:            p = "MQAT_JAVA";            break;
    case MQAT_DQM:             p = "MQAT_DQM";             break;
    default:
      sprintf(Buffer, "%d", ApplType);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an ExitId value into a string                             */
/*                                                                   */
/*********************************************************************/

char * strExitId ( MQLONG ExitId, char *Buffer )
{
  char *p = NULL;

  switch (ExitId)
  {
    case MQXT_API_EXIT:               p = "MQXT_API_EXIT";               break;
    case MQXT_CHANNEL_SEC_EXIT:       p = "MQXT_CHANNEL_SEC_EXIT";       break;
    case MQXT_CHANNEL_MSG_EXIT:       p = "MQXT_CHANNEL_MSG_EXIT";       break;
    case MQXT_CHANNEL_SEND_EXIT:      p = "MQXT_CHANNEL_SEND_EXIT";      break;
    case MQXT_CHANNEL_RCV_EXIT:       p = "MQXT_CHANNEL_RCV_EXIT";       break;
    case MQXT_CHANNEL_MSG_RETRY_EXIT: p = "MQXT_CHANNEL_MSG_RETRY_EXIT"; break;
    case MQXT_CHANNEL_AUTO_DEF_EXIT:  p = "MQXT_CHANNEL_AUTO_DEF_EXIT";  break;
    case MQXT_CLUSTER_WORKLOAD_EXIT:  p = "MQXT_CLUSTER_WORKLOAD_EXIT";  break;
    case MQXT_PUBSUB_ROUTING_EXIT:    p = "MQXT_PUBSUB_ROUTING_EXIT";    break;
    default:
      sprintf(Buffer, "%d", ExitId);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an ExitReason into a string                               */
/*                                                                   */
/*********************************************************************/

char * strExitReason ( MQLONG ExitReason, char *Buffer )
{
  char *p = NULL;

  switch (ExitReason)
  {
    case MQXR_BEFORE:           p = "MQXR_BEFORE";          break;
    case MQXR_AFTER:            p = "MQXR_AFTER";           break;
    case MQXR_CONNECTION:       p = "MQXR_CONNECTION";      break;
    case MQXR_INIT:             p = "MQXR_INIT";            break;
    case MQXR_TERM:             p = "MQXR_TERM";            break;
    case MQXR_MSG:              p = "MQXR_MSG";             break;
    case MQXR_XMIT:             p = "MQXR_XMIT";            break;
    case MQXR_SEC_MSG:          p = "MQXR_SEC_MSG";         break;
    case MQXR_INIT_SEC:         p = "MQXR_INIT_SEC";        break;
    case MQXR_RETRY:            p = "MQXR_RETRY";           break;
    case MQXR_AUTO_CLUSSDR:     p = "MQXR_AUTO_CLUSSDR";    break;
    case MQXR_AUTO_RECEIVER:    p = "MQXR_AUTO_RECEIVER";   break;
    case MQXR_CLWL_OPEN:        p = "MQXR_CLWL_OPEN";       break;
    case MQXR_CLWL_PUT:         p = "MQXR_CLWL_PUT";        break;
    case MQXR_CLWL_MOVE:        p = "MQXR_CLWL_MOVE";       break;
    case MQXR_CLWL_REPOS:       p = "MQXR_CLWL_REPOS";      break;
    case MQXR_CLWL_REPOS_MOVE:  p = "MQXR_CLWL_REPOS_MOVE"; break;

#if (MQAT_DEFAULT == MQAT_UNIX)
    case MQXR_END_BATCH:        p = "MQXR_END_BATCH";       break;
    case MQXR_ACK_RECEIVED:     p = "MQXR_ACK_RECEIVED";    break;
#endif

    case MQXR_AUTO_SVRCONN:     p = "MQXR_AUTO_SVRCONN";    break;
    case MQXR_AUTO_CLUSRCVR:    p = "MQXR_AUTO_CLUSRCVR";   break;
    default:
      sprintf(Buffer, "%d", ExitReason);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an ExitResponse into a string                             */
/*                                                                   */
/*********************************************************************/

char * strExitResponse ( MQLONG ExitResponse, char *Buffer )
{
  char *p = NULL;

  switch (ExitResponse)
  {
    case MQXCC_OK:                       p = "MQXCC_OK";                       break;
    case MQXCC_SUPPRESS_FUNCTION:        p = "MQXCC_SUPPRESS_FUNCTION";        break;
    case MQXCC_SKIP_FUNCTION:            p = "MQXCC_SKIP_FUNCTION";            break;
    case MQXCC_SEND_AND_REQUEST_SEC_MSG: p = "MQXCC_SEND_AND_REQUEST_SEC_MSG"; break;
    case MQXCC_SEND_SEC_MSG:             p = "MQXCC_SEND_SEC_MSG";             break;
    case MQXCC_SUPPRESS_EXIT:            p = "MQXCC_SUPPRESS_EXIT";            break;
    case MQXCC_CLOSE_CHANNEL:            p = "MQXCC_CLOSE_CHANNEL";            break;

#if (MQAT_DEFAULT == MQAT_UNIX)
    case MQXCC_REQUEST_ACK:              p = "MQXCC_REQUEST_ACK";              break;
#endif

    case MQXCC_FAILED:                   p = "MQXCC_FAILED";                   break;
    default:
      sprintf(Buffer, "%d", ExitResponse);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an ExitResponse2 into a string                            */
/*                                                                   */
/*********************************************************************/

char * strExitResponse2 ( MQLONG ExitResponse2, char *Buffer )
{
  char *p = NULL;

  switch (ExitResponse2)
  {
    case MQXR2_DEFAULT_CONTINUATION: p = "MQXR2_DEFAULT_CONTINUATION"; break;
    case MQXR2_CONTINUE_CHAIN:       p = "MQXR2_CONTINUE_CHAIN";       break;
    case MQXR2_SUPPRESS_CHAIN:       p = "MQXR2_SUPPRESS_CHAIN";       break;
    default:
      sprintf(Buffer, "%d", ExitResponse2);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an APICallerType into a string                            */
/*                                                                   */
/*********************************************************************/

char * strAPICallerType ( MQLONG ApiCallerType, char *Buffer )
{
  char *p = NULL;

  switch (ApiCallerType)
  {
    case MQXACT_EXTERNAL:   p = "MQXACT_EXTERNAL";   break;
    case MQXACT_INTERNAL:   p = "MQXACT_INTERNAL";   break;
    default:
      sprintf(Buffer, "%d", ApiCallerType);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an Function Id into a string                              */
/*                                                                   */
/*********************************************************************/

char * strFunctionId ( MQLONG Function, char *Buffer )
{
  char *p = NULL;

  switch (Function)
  {
    case MQXF_INIT:             p = "MQXF_INIT";             break;
    case MQXF_TERM:             p = "MQXF_TERM";             break;
    case MQXF_CONN:             p = "MQXF_CONN";             break;
    case MQXF_CONNX:            p = "MQXF_CONNX";            break;
    case MQXF_DISC:             p = "MQXF_DISC";             break;
    case MQXF_OPEN:             p = "MQXF_OPEN";             break;
    case MQXF_CLOSE:            p = "MQXF_CLOSE";            break;
    case MQXF_PUT1:             p = "MQXF_PUT1";             break;
    case MQXF_PUT:              p = "MQXF_PUT";              break;
    case MQXF_GET:              p = "MQXF_GET";              break;
    case MQXF_DATA_CONV_ON_GET: p = "MQXF_DATA_CONV_ON_GET"; break;
    case MQXF_INQ:              p = "MQXF_INQ";              break;
    case MQXF_SET:              p = "MQXF_SET";              break;
    case MQXF_BEGIN:            p = "MQXF_BEGIN";            break;
    case MQXF_CMIT:             p = "MQXF_CMIT";             break;
    case MQXF_BACK:             p = "MQXF_BACK";             break;
    default:
      sprintf(Buffer, "%d", Function);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Convert an Feedback value into a string                           */
/*                                                                   */
/*********************************************************************/

char * strFeedback ( MQLONG Feedback, char *Buffer )
{
  char *p = NULL;

  switch (Feedback)
  {
    case MQFB_NONE:                   p = "MQFB_NONE";                   break;
    case MQFB_QUIT:                   p = "MQFB_QUIT";                   break;
    case MQFB_EXPIRATION:             p = "MQFB_EXPIRATION";             break;
    case MQFB_COA:                    p = "MQFB_COA";                    break;
    case MQFB_COD:                    p = "MQFB_COD";                    break;
    case MQFB_CHANNEL_COMPLETED:      p = "MQFB_CHANNEL_COMPLETED";      break;
    case MQFB_CHANNEL_FAIL_RETRY:     p = "MQFB_CHANNEL_FAIL_RETRY";     break;
    case MQFB_CHANNEL_FAIL:           p = "MQFB_CHANNEL_FAIL";           break;
    case MQFB_APPL_CANNOT_BE_STARTED: p = "MQFB_APPL_CANNOT_BE_STARTED"; break;
    case MQFB_TM_ERROR:               p = "MQFB_TM_ERROR";               break;
    case MQFB_APPL_TYPE_ERROR:        p = "MQFB_APPL_TYPE_ERROR";        break;
    case MQFB_STOPPED_BY_MSG_EXIT:    p = "MQFB_STOPPED_BY_MSG_EXIT";    break;
    case MQFB_XMIT_Q_MSG_ERROR:       p = "MQFB_XMIT_Q_MSG_ERROR";       break;
    case MQFB_PAN:                    p = "MQFB_PAN";                    break;
    case MQFB_NAN:                    p = "MQFB_NAN";                    break;
    case MQFB_STOPPED_BY_CHAD_EXIT:   p = "MQFB_STOPPED_BY_CHAD_EXIT";   break;
    case MQFB_STOPPED_BY_PUBSUB_EXIT: p = "MQFB_STOPPED_BY_PUBSUB_EXIT"; break;
    case MQFB_NOT_A_REPOSITORY_MSG:   p = "MQFB_NOT_A_REPOSITORY_MSG";   break;
    case MQFB_BIND_OPEN_CLUSRCVR_DEL: p = "MQFB_BIND_OPEN_CLUSRCVR_DEL"; break;
    case MQFB_DATA_LENGTH_ZERO:       p = "MQFB_DATA_LENGTH_ZERO";       break;
    case MQFB_DATA_LENGTH_NEGATIVE:   p = "MQFB_DATA_LENGTH_NEGATIVE";   break;
    case MQFB_DATA_LENGTH_TOO_BIG:    p = "MQFB_DATA_LENGTH_TOO_BIG";    break;
    case MQFB_BUFFER_OVERFLOW:        p = "MQFB_BUFFER_OVERFLOW";        break;
    case MQFB_LENGTH_OFF_BY_ONE:      p = "MQFB_LENGTH_OFF_BY_ONE";      break;
    case MQFB_IIH_ERROR:              p = "MQFB_IIH_ERROR";              break;
    case MQFB_NOT_AUTHORIZED_FOR_IMS: p = "MQFB_NOT_AUTHORIZED_FOR_IMS"; break;
    case MQFB_IMS_ERROR:              p = "MQFB_IMS_ERROR";              break;
    case MQFB_IMS_FIRST:              p = "MQFB_IMS_FIRST";              break;
    case MQFB_IMS_LAST:               p = "MQFB_IMS_LAST";               break;
    case MQFB_CICS_INTERNAL_ERROR:    p = "MQFB_CICS_INTERNAL_ERROR";    break;
    case MQFB_CICS_NOT_AUTHORIZED:    p = "MQFB_CICS_NOT_AUTHORIZED";    break;
    case MQFB_CICS_BRIDGE_FAILURE:    p = "MQFB_CICS_BRIDGE_FAILURE";    break;
    case MQFB_CICS_CORREL_ID_ERROR:   p = "MQFB_CICS_CORREL_ID_ERROR";   break;
    case MQFB_CICS_CCSID_ERROR:       p = "MQFB_CICS_CCSID_ERROR";       break;
    case MQFB_CICS_ENCODING_ERROR:    p = "MQFB_CICS_ENCODING_ERROR";    break;
    case MQFB_CICS_CIH_ERROR:         p = "MQFB_CICS_CIH_ERROR";         break;
    case MQFB_CICS_UOW_ERROR:         p = "MQFB_CICS_UOW_ERROR";         break;
    case MQFB_CICS_COMMAREA_ERROR:    p = "MQFB_CICS_COMMAREA_ERROR";    break;
    case MQFB_CICS_APPL_NOT_STARTED:  p = "MQFB_CICS_APPL_NOT_STARTED";  break;
    case MQFB_CICS_APPL_ABENDED:      p = "MQFB_CICS_APPL_ABENDED";      break;
    case MQFB_CICS_DLQ_ERROR:         p = "MQFB_CICS_DLQ_ERROR";         break;
    case MQFB_CICS_UOW_BACKED_OUT:    p = "MQFB_CICS_UOW_BACKED_OUT";    break;
    default:
      sprintf(Buffer, "%d", Feedback);
      p = Buffer;
  }

  return p;
}

/*********************************************************************/
/*                                                                   */
/* Check to see if a string is blank                                 */
/*                                                                   */
/*********************************************************************/

MQLONG myBlankCheck ( void * DataPointer, size_t DataLength )
{
  size_t i      = 0;
  MQLONG result = 1;
  MQLONG more   = 1;
  char * string = DataPointer;

  for (i = 0; more && (i < DataLength); i++)
  {
    switch (string[i])
    {
      case 0:
        more   = 0;
        break;

      case ' ':
        break;

      default:
        result = 0;
        more   = 0;
        break;
    }
  }

  return result;
}

/*********************************************************************/
/*                                                                   */
/* Check a pointer is valid                                          */
/*                                                                   */
/*********************************************************************/

MQLONG myCheckPointer ( void * DataPointer, size_t DataLength )
{
  if (DataPointer)
    return 1;
  else
    return 0;
}

/*********************************************************************/
/*                                                                   */
/* Return a string representation of a pointer                       */
/*                                                                   */
/*********************************************************************/

char * strptr ( void *Pointer, char *format, char * StringBuffer )
{
  if (myCheckPointer( Pointer, sizeof(void*)))
  {
    void **pPointer = Pointer;

    sprintf( StringBuffer, format, *pPointer);
    return StringBuffer;
  }
  else
    return "(null)";
}

/*********************************************************************/
/*                                                                   */
/* Return a string representation of a pointer                       */
/*                                                                   */
/*********************************************************************/

char * strpptr ( void *Pointer, char *format, char * StringBuffer )
{
  if (myCheckPointer( Pointer, sizeof(void*)))
  {
    void **pPointer = Pointer;
    return strptr( *pPointer, format, StringBuffer );
  }
  else
    return "(null)";
}

/*********************************************************************/
/*                                                                   */
/* Initialise the Windows Performance counter                        */
/*                                                                   */
/*********************************************************************/

#if (MQAT_DEFAULT == MQAT_WINDOWS_NT)

int InitPerformanceCounter( MYEXITUSERAREA * pExitUserArea )
{
  LARGE_INTEGER PerformanceFrequency;
  QueryPerformanceFrequency( &PerformanceFrequency );
  pExitUserArea->Frequency = (double) PerformanceFrequency.QuadPart;

  QueryPerformanceCounter( &pExitUserArea->PerformanceCounter );

  return 0;
}

#endif

/*********************************************************************/
/*                                                                   */
/* Make a timestamp string using the current time                    */
/*                                                                   */
/*********************************************************************/

void myGetAbsoluteTime ( char * StringBuffer, size_t StringBufferLength )
{
  time_t long_time;
  struct tm *newtime;

  time( &long_time );
  newtime = localtime( &long_time );

  strftime( StringBuffer, StringBufferLength, "%Y-%m-%d  %H:%M:%S", newtime );
}

void myGetRelativeTime ( char           * StringBuffer
                       , MYEXITUSERAREA * pExitUserArea
                       )
{
#if (MQAT_DEFAULT == MQAT_WINDOWS_NT)
  LARGE_INTEGER PerformanceCounter;
  double        Counter;
  double        Period;

  QueryPerformanceCounter( &PerformanceCounter ) ;

  Counter   = (double) ( PerformanceCounter.QuadPart - pExitUserArea->PerformanceCounter.QuadPart );
  Period    = Counter * 1000000.0 / pExitUserArea->Frequency;

  sprintf( StringBuffer, "%.0lf", Period );
#endif

#if (MQAT_DEFAULT == MQAT_UNIX) || (MQAT_DEFAULT == MQAT_OS400)
  struct timeval t1;
  struct timezone tz ;
  gettimeofday(&t1, &tz );
  sprintf( StringBuffer, "%ld.%ld", t1.tv_sec, t1.tv_usec );
#endif
}

/*********************************************************************/
/*                                                                   */
/* Format a block of data into hex and dump to a file                */
/*                                                                   */
/* Offset    Data                                                    */
/* XXXXXXXX: XXXXXXXX XXXXXXXX XXXXXXXX XXXXXXXX   ................  */
/* 0         1         2         3         4         5         6         7   */
/* 012345678901234567890123456789012345678901234567890123456789012345678901  */
/*                                                                   */
/*********************************************************************/

char HEX[] = "0123456789ABCDEF";

void DumpHex( FILE   * fp
            , void   * DataPointer
            , size_t   DataLength
            )
{
  if (myCheckPointer(DataPointer, DataLength))
  {
    char             StringBuffer[100];
    unsigned char  * DataBuffer = DataPointer;
    unsigned char  * p          = DataPointer;
    unsigned char  * pd;
    unsigned char  * pa;
    size_t           PointerSize = sizeof(void*);
    size_t           DataOffset  = (PointerSize * 2) + 2;
    size_t           AsciiOffset = DataOffset  + 38;
    size_t           LineLength  = AsciiOffset + 16;
    size_t           index, i;
    int              n;

    sprintf( &StringBuffer[ LineLength ], "\n" );

    index = 0;
    while (index < DataLength)
    {
      pd = (unsigned char*) StringBuffer + DataOffset;
      pa = (unsigned char*) StringBuffer + AsciiOffset;

      memset( StringBuffer, ' ', LineLength);
      n = sprintf(StringBuffer, "%p", p);
      StringBuffer[ n ] = ':';

      for (i = 0; (i < 16) && (index < DataLength); i++)
      {
        if ((i % 4) == 0) pd++;

        *pd++ = HEX[ *p / 16 ];
        *pd++ = HEX[ *p % 16 ];
        *pa++ = isprint(*p) ? *p : '.';

        index++;
        p++;
      }

      fprintf(fp, "          %s", StringBuffer);
    }
  }
}

/*********************************************************************/
/*                                                                   */
/* Dump the ExitChainAreaPtr                                         */
/*                                                                   */
/*********************************************************************/

void DumpExitChainArea( FILE * fp, PMQAXP pExitParms )
{
  MYEXITUSERAREA ** ppExitUserArea   = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea    = *ppExitUserArea;
  MQACH           * ExitChainAreaPtr = pExitParms->ExitChainAreaPtr;

  if (  (pExitUserArea->Options & OPTIONS_DUMP_EXITCHAINAREA)
     && (ExitChainAreaPtr)
     )
  {
    fprintf( fp, "   --------------------------\n" );
    fprintf( fp, "   ExitChainAreaPtr:       %p\n",    ExitChainAreaPtr );

    while (ExitChainAreaPtr)
    {
      fprintf( fp, "    StrucId:               %.4s\n",  ExitChainAreaPtr->StrucId );

      if (memcmp( ExitChainAreaPtr->StrucId, MQACH_STRUC_ID, 4))
      {
        fprintf( fp, "ERROR *** BAD MQACH StrucId ***\n" );
        ExitChainAreaPtr = NULL;
      }
      else
      {
        fprintf( fp, "    Version:               %d\n",    ExitChainAreaPtr->Version          );
        fprintf( fp, "    StrucLength:           %d\n",    ExitChainAreaPtr->StrucLength      );
        fprintf( fp, "    ChainAreaLength:       %d\n",    ExitChainAreaPtr->ChainAreaLength  );
        fprintf( fp, "    ExitInfoName:          %.48s\n", ExitChainAreaPtr->ExitInfoName     );
        fprintf( fp, "    NextChainAreaPtr:      %p\n",    ExitChainAreaPtr->NextChainAreaPtr );

        ExitChainAreaPtr = ExitChainAreaPtr->NextChainAreaPtr;

        fprintf( fp, "   --------------------------\n" );
        fprintf( fp, "   ExitChainAreaPtr:       %p\n",    ExitChainAreaPtr );
      }
    }

    fprintf( fp, "   --------------------------\n" );
  }
  else
    fprintf( fp, "   ExitChainAreaPtr:       %p\n",    ExitChainAreaPtr );
}

/*********************************************************************/
/*                                                                   */
/* Dump the ApiExit Params                                           */
/*                                                                   */
/*********************************************************************/

void MQENTRY DumpParms ( FILE *fp, PMQAXP pExitParms )
{
  char buffer[50] = "";

  fprintf( fp, "  PARMS\n" );
  fprintf( fp, "   ExitId:                 %s\n",    strExitId( pExitParms->ExitId, buffer ) );
  fprintf( fp, "   ExitReason:             %s\n",    strExitReason( pExitParms->ExitReason, buffer ) );
  fprintf( fp, "   ExitResponse:           %s\n",    strExitResponse( pExitParms->ExitResponse, buffer ) );
  fprintf( fp, "   ExitResponse2:          %s\n",    strExitResponse2( pExitParms->ExitResponse2, buffer ) );
  fprintf( fp, "   Feedback:               %s\n",    strFeedback( pExitParms->Feedback, buffer ) );
  fprintf( fp, "   APICallerType:          %s\n",    strAPICallerType( pExitParms->APICallerType, buffer ) );
  fprintf( fp, "   ExitUserArea:           %p\n",    pExitParms->ExitUserArea );
  DumpHex(fp, pExitParms->ExitUserArea, MQ_EXIT_USER_AREA_LENGTH);
  fprintf( fp, "   ExitData:               %.32s\n", pExitParms->ExitData );
  fprintf( fp, "   ExitInfoName:           %.48s\n", pExitParms->ExitInfoName );
  fprintf( fp, "   ExitPDArea:             %p\n",    pExitParms->ExitPDArea );
  DumpHex(fp, pExitParms->ExitPDArea, MQ_EXIT_PD_AREA_LENGTH);
  fprintf( fp, "   QMgrName:               %.48s\n", pExitParms->QMgrName );
  DumpExitChainArea( fp, pExitParms );
  fprintf( fp, "   Hconfig:                %p\n",    pExitParms->Hconfig );
  fprintf( fp, "   Function:               %s\n",    strFunctionId( pExitParms->Function, buffer ) );
}

/*********************************************************************/
/*                                                                   */
/* Dump the ApiExit Context                                          */
/*                                                                   */
/*********************************************************************/

void MQENTRY DumpContext ( FILE *fp, PMQAXC pExitContext )
{
  char buffer[50] = "";

  fprintf( fp, "  CONTEXT\n" );
  fprintf( fp, "   Environment:            %s\n", strEnvironment( pExitContext->Environment, buffer ) );
  fprintf( fp, "   UserId:                 %.12s\n", pExitContext->UserId );
  fprintf( fp, "   SecurityId:\n" );
  DumpHex(fp, pExitContext->SecurityId, MQ_SECURITY_ID_LENGTH);
  fprintf( fp, "   ConnectionName:         %.264s\n", pExitContext->ConnectionName );
  fprintf( fp, "   LongMCAUserIdLength:    %d\n", pExitContext->LongMCAUserIdLength );
  DumpHex(fp, pExitContext->LongMCAUserIdPtr, pExitContext->LongMCAUserIdLength);
  fprintf( fp, "   LongRemoteUserIdLength: %d\n", pExitContext->LongRemoteUserIdLength );
  DumpHex(fp, pExitContext->LongRemoteUserIdPtr, pExitContext->LongRemoteUserIdLength);
  fprintf( fp, "   ApplName:               %.28s\n", pExitContext->ApplName );
  fprintf( fp, "   ApplType:               %s\n", strApplType( pExitContext->ApplType, buffer ) );
  fprintf( fp, "   ProcessId:              %d\n", pExitContext->ProcessId );
  fprintf( fp, "   ThreadId:               %d\n", pExitContext->ThreadId );
}

/*********************************************************************/
/*                                                                   */
/* Before MQBACK Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_BACK_EXIT BackBefore;

void MQENTRY BackBefore  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQBACK\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn, "0x%X", buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQBACK Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_BACK_EXIT BackAfter;

void MQENTRY BackAfter  ( PMQAXP    pExitParms
                        , PMQAXC    pExitContext
                        , PMQHCONN  pHconn
                        , PMQLONG   pCompCode
                        , PMQLONG   pReason
                        )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQBAGIN Entrypoint                                         */
/*                                                                   */
/*********************************************************************/

MQ_BEGIN_EXIT BeginBefore;

void MQENTRY BeginBefore ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQBO    ppBeginOptions
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQBEGIN\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn, "0x%X", buffer1));
  fprintf(fp, "  BeginOptions  : %p\n",   *ppBeginOptions );
  DumpHex(fp, *ppBeginOptions, sizeof(MQBO));

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQBEGIN Entrypoint                                         */
/*                                                                   */
/*********************************************************************/

MQ_BEGIN_EXIT BeginAfter;

void MQENTRY BeginAfter  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQBO    ppBeginOptions
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  BeginOptions  : %p\n", *ppBeginOptions );
  DumpHex(fp, *ppBeginOptions, sizeof(MQBO));
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQCLOSE Entrypoint                                         */
/*                                                                   */
/*********************************************************************/

MQ_CLOSE_EXIT CloseBefore;

void MQENTRY CloseBefore ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQHOBJ  ppHobj
                         , PMQLONG   pOptions
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQCLOSE\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn, "0x%X", buffer1));
  fprintf(fp, "  Hobj          : %s\n", strpptr(ppHobj, "0x%X", buffer1));
  fprintf(fp, "  Options       : %d\n", *pOptions  );

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQCLOSE Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_CLOSE_EXIT CloseAfter;

void MQENTRY CloseAfter  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQHOBJ  ppHobj
                         , PMQLONG   pOptions
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hobj          : %s\n", strpptr(ppHobj, "0x%X", buffer1) );
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQCMIT Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_CMIT_EXIT CmitBefore;

void MQENTRY CmitBefore  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQCMIT\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn, "0x%X", buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQCMIT Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_CMIT_EXIT CmitAfter;

void MQENTRY CmitAfter   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQCONN Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_CONNX_EXIT ConnBefore;

void MQENTRY ConnBefore  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , MQCHAR48  QMgrName
                         , PPMQCNO   ppConnectOpts
                         , PPMQHCONN ppHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQCONN\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  QMgrName      : %.48s\n", QMgrName      );
  fprintf(fp, "  ConnectOpts   : 0x%p\n", *ppConnectOpts );
  DumpHex(fp, *ppConnectOpts, sizeof(MQCNO));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQCONN Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_CONNX_EXIT ConnAfter;

void MQENTRY ConnAfter   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , MQCHAR48  QMgrName
                         , PPMQCNO   ppConnectOpts
                         , PPMQHCONN ppHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strpptr(ppHconn, "0x%X", buffer1));
  fprintf(fp, "  CompCode      : %s\n", strptr (pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr (pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQCONNX Entrypoint                                         */
/*                                                                   */
/*********************************************************************/

MQ_CONNX_EXIT ConnxBefore;

void MQENTRY ConnxBefore ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , MQCHAR48  QMgrName
                         , PPMQCNO   ppConnectOpts
                         , PPMQHCONN ppHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQCONNX\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  QMgrName      : %.48s\n", QMgrName      );
  fprintf(fp, "  ConnectOpts   : 0x%p\n", *ppConnectOpts );
  DumpHex(fp, *ppConnectOpts, sizeof(MQCNO));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQCONNX Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_CONNX_EXIT ConnxAfter;

void MQENTRY ConnxAfter  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , MQCHAR48  QMgrName
                         , PPMQCNO   ppConnectOpts
                         , PPMQHCONN ppHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strpptr(ppHconn,   "0x%X", buffer1));
  fprintf(fp, "  CompCode      : %s\n", strptr (pCompCode, "%d",   buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr (pReason,   "%d",   buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQDISC Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_DISC_EXIT DiscBefore;

void MQENTRY DiscBefore  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PPMQHCONN ppHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQDISC\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strpptr(ppHconn, "0x%X", buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQDISC Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_DISC_EXIT DiscAfter;

void MQENTRY DiscAfter   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PPMQHCONN ppHconn
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strpptr(ppHconn,   "0x%X", buffer1));
  fprintf(fp, "  CompCode      : %s\n", strptr (pCompCode, "%d",   buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr (pReason,   "%d",   buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQGET Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_GET_EXIT GetBefore;

void MQENTRY GetBefore   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PPMQMD    ppMsgDesc
                         , PPMQGMO   ppGetMsgOpts
                         , PMQLONG   pBufferLength
                         , PPMQVOID  ppBuffer
                         , PPMQLONG  ppDataLength
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQGET\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn,       "0x%X", buffer1));
  fprintf(fp, "  Hobj          : %s\n", strptr(pHobj,        "0x%X", buffer1));
  fprintf(fp, "  MsgDesc       : %s\n", strptr(ppMsgDesc,    "0x%p",   buffer1));
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  GetMsgOpts    : %s\n", strptr(ppGetMsgOpts, "0x%p",   buffer1));
  DumpHex(fp, *ppGetMsgOpts, sizeof(MQGMO));
  fprintf(fp, "  BufferLength  : %s\n", strptr(pBufferLength, "0x%p",  buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before Data Conversion on MQGET Entrypoint                        */
/*                                                                   */
/*********************************************************************/

MQ_GET_EXIT GetBeforeDataConv;

void MQENTRY GetBeforeDataConv( PMQAXP    pExitParms
                              , PMQAXC    pExitContext
                              , PMQHCONN  pHconn
                              , PMQHOBJ   pHobj
                              , PPMQMD    ppMsgDesc
                              , PPMQGMO   ppGetMsgOpts
                              , PMQLONG   pBufferLength
                              , PPMQVOID  ppBuffer
                              , PPMQLONG  ppDataLength
                              , PMQLONG   pCompCode
                              , PMQLONG   pReason
                              )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "BEFORE DATA CONVERSION", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  MsgDesc       : %s\n", strptr(ppMsgDesc,     "0x%p", buffer1));
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  GetMsgOpts    : %s\n", strptr(ppGetMsgOpts,  "0x%p", buffer1));
  DumpHex(fp, *ppGetMsgOpts, sizeof(MQGMO));
  fprintf(fp, "  DataLength    : %s\n", strpptr(ppDataLength, "0x%p", buffer1));
  fprintf(fp, "  Buffer        : %s\n", strptr(ppBuffer,  "0x%p", buffer1));
  DumpHex(fp, *ppBuffer, min(1024, **ppDataLength));
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQGET Entrypoint                                            */
/*                                                                   */
/*********************************************************************/

MQ_GET_EXIT GetAfter;

void MQENTRY GetAfter    ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PPMQMD    ppMsgDesc
                         , PPMQGMO   ppGetMsgOpts
                         , PMQLONG   pBufferLength
                         , PPMQVOID  ppBuffer
                         , PPMQLONG  ppDataLength
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  MsgDesc       : %s\n", strptr(ppMsgDesc,     "0x%p", buffer1));
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  GetMsgOpts    : %s\n", strptr(ppGetMsgOpts,  "0x%p", buffer1));
  DumpHex(fp, *ppGetMsgOpts, sizeof(MQGMO));
  fprintf(fp, "  DataLength    : %s\n", strpptr(ppDataLength, "0x%p", buffer1));
  fprintf(fp, "  Buffer        : %s\n", strptr(ppBuffer,  "0x%p", buffer1));
  DumpHex(fp, *ppBuffer, min(1024, **ppDataLength));
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQINQ Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_INQ_EXIT InqBefore;

void MQENTRY InqBefore   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PMQLONG   pSelectorCount
                         , PPMQLONG  ppSelectors
                         , PMQLONG   pIntAttrCount
                         , PPMQLONG  ppIntAttrs
                         , PMQLONG   pCharAttrLength
                         , PPMQCHAR  ppCharAttrs
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQINQ\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn, "0x%X", buffer1));
  fprintf(fp, "  Hobj          : %s\n", strptr(pHobj,  "0x%X", buffer1));
  fprintf(fp, "  SelectorCount : %d\n", *pSelectorCount   );
  fprintf(fp, "  Selectors     : %s\n", strpptr(ppSelectors,    "0x%p", buffer1));
  DumpHex(fp, *ppSelectors, *pSelectorCount * sizeof(MQLONG) );
  fprintf(fp, "  IntAttrCount  : %s\n", strptr(pIntAttrCount,   "%d", buffer1));
  fprintf(fp, "  CharAttrLength: %s\n", strptr(pCharAttrLength, "%d", buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQINQ Entrypoint                                            */
/*                                                                   */
/*********************************************************************/

MQ_INQ_EXIT InqAfter;

void MQENTRY InqAfter    ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PMQLONG   pSelectorCount
                         , PPMQLONG  ppSelectors
                         , PMQLONG   pIntAttrCount
                         , PPMQLONG  ppIntAttrs
                         , PMQLONG   pCharAttrLength
                         , PPMQCHAR  ppCharAttrs
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  IntAttrs      : %s\n", strptr(ppIntAttrs,  "0x%p", buffer1));
  DumpHex(fp, *ppIntAttrs, *pIntAttrCount * sizeof(MQLONG) );
  fprintf(fp, "  CharAttrs     : %s\n", strptr(ppCharAttrs, "0x%p", buffer1));
  DumpHex(fp, *ppCharAttrs, *pCharAttrLength               );
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQOPEN Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_OPEN_EXIT OpenBefore;

void MQENTRY OpenBefore  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQOD    ppObjDesc
                         , PMQLONG   pOptions
                         , PPMQHOBJ  ppHobj
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQOPEN\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n",   strptr(pHconn,    "0x%X", buffer1));
  fprintf(fp, "  pObjDesc      : %s\n",   strptr(ppObjDesc, "0x%p",   buffer1));
  DumpHex(fp, *ppObjDesc, sizeof(MQOD));
  fprintf(fp, "  Options       : %s\n", strptr(pOptions,  "0x%X", buffer1));
  fprintf(fp, "  pHobj         : %s\n",   strptr(ppHobj,    "0x%X",   buffer1));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQOPEN Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_OPEN_EXIT OpenAfter;

void MQENTRY OpenAfter   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQOD    ppObjDesc
                         , PMQLONG   pOptions
                         , PPMQHOBJ  ppHobj
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  pObjDesc      : %s\n", strptr(ppObjDesc, "0x%p",  buffer1));
  DumpHex(fp, *ppObjDesc, sizeof(MQOD));
  fprintf(fp, "  Hobj          : %s\n", strpptr(ppHobj,   "0x%X", buffer1));
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQPUT Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_PUT_EXIT PutBefore;

void MQENTRY PutBefore   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PPMQMD    ppMsgDesc
                         , PPMQPMO   ppPutMsgOpts
                         , PMQLONG   pBufferLength
                         , PPMQVOID  ppBuffer
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQPUT\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn,        "0x%X", buffer1));
  fprintf(fp, "  Hobj          : %s\n", strptr(pHobj,         "0x%X", buffer1));
  fprintf(fp, "  MsgDesc       : %s\n", strptr(ppMsgDesc,     "0x%p",   buffer1));
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  PutMsgOpts    : %s\n", strptr(ppPutMsgOpts,  "0x%p",   buffer1));
  DumpHex(fp, *ppPutMsgOpts, sizeof(MQPMO));
  fprintf(fp, "  BufferLength  : %s\n", strptr(pBufferLength, "%d",   buffer1));
  fprintf(fp, "  Buffer        : %s\n", strptr(ppBuffer,      "0x%p",   buffer1));

  if (myCheckPointer(pBufferLength, sizeof(void*)))
    DumpHex(fp, *ppBuffer, min(1024, *pBufferLength));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQPUT Entrypoint                                            */
/*                                                                   */
/*********************************************************************/

MQ_PUT_EXIT PutAfter;

void MQENTRY PutAfter    ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PPMQMD    ppMsgDesc
                         , PPMQPMO   ppPutMsgOpts
                         , PMQLONG   pBufferLength
                         , PPMQVOID  ppBuffer
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  MsgDesc       : %s\n", strptr(ppMsgDesc,    "0x%p", buffer1));
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  PutMsgOpts    : %s\n", strptr(ppPutMsgOpts, "0x%p", buffer1));
  DumpHex(fp, *ppPutMsgOpts, sizeof(MQPMO));
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode,    "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,      "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQPUT1 Entrypoint                                          */
/*                                                                   */
/*********************************************************************/

MQ_PUT1_EXIT Put1Before;

void MQENTRY Put1Before  ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQOD    ppObjDesc
                         , PPMQMD    ppMsgDesc
                         , PPMQPMO   ppPut1MsgOpts
                         , PMQLONG   pBufferLength
                         , PPMQVOID  ppBuffer
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQPUT1\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn,        "0x%X", buffer1));
  fprintf(fp, "  pObjDesc      : %s\n", strptr(ppObjDesc,     "0x%p",   buffer1));
  DumpHex(fp, *ppObjDesc, sizeof(MQOD));
  fprintf(fp, "  MsgDesc       : %s\n", strptr(ppMsgDesc,     "0x%p",   buffer1));
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  Put1MsgOpts    : %s\n",strptr(ppPut1MsgOpts, "0x%p",   buffer1));
  DumpHex(fp, *ppPut1MsgOpts, sizeof(MQPMO));
  fprintf(fp, "  BufferLength  : %s\n", strptr(pBufferLength, "%d",   buffer1));
  fprintf(fp, "  Buffer        : %s\n", strptr(ppBuffer,      "0x%p",   buffer1));

  if (myCheckPointer(pBufferLength, sizeof(void*)))
    DumpHex(fp, *ppBuffer, min(1024, *pBufferLength));

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQPUT1 Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_PUT1_EXIT Put1After;

void MQENTRY Put1After   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PPMQOD    ppObjDesc
                         , PPMQMD    ppMsgDesc
                         , PPMQPMO   ppPut1MsgOpts
                         , PMQLONG   pBufferLength
                         , PPMQVOID  ppBuffer
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  pObjDesc      : 0x%p\n",   *ppObjDesc      );
  DumpHex(fp, *ppObjDesc, sizeof(MQOD));
  fprintf(fp, "  MsgDesc       : 0x%p\n",   *ppMsgDesc      );
  DumpHex(fp, *ppMsgDesc, sizeof(MQMD));
  fprintf(fp, "  Put1MsgOpts    : 0x%p\n",  *ppPut1MsgOpts );
  DumpHex(fp, *ppPut1MsgOpts, sizeof(MQPMO));
  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* Before MQSET Entrypoint                                           */
/*                                                                   */
/*********************************************************************/

MQ_SET_EXIT SetBefore;

void MQENTRY SetBefore   ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PMQLONG   pSelectorCount
                         , PPMQLONG  ppSelectors
                         , PMQLONG   pIntAttrCount
                         , PPMQLONG  ppIntAttrs
                         , PMQLONG   pCharAttrLength
                         , PPMQCHAR  ppCharAttrs
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQSET\n");
  fprintf(fp, TITLE_FORMAT, "BEFORE", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_BEFORE)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_BEFORE)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  Hconn         : %s\n", strptr(pHconn,          "0x%X",   buffer1));
  fprintf(fp, "  Hobj          : %s\n", strptr(pHobj,           "0x%X", buffer1));
  fprintf(fp, "  SelectorCount : %s\n", strptr(pSelectorCount,  "%d",   buffer1));
  fprintf(fp, "  Selectors     : %s\n", strptr(ppSelectors,     "0x%p",   buffer1));
  DumpHex(fp, *ppSelectors, *pSelectorCount * sizeof(MQLONG));
  fprintf(fp, "  IntAttrCount  : %s\n", strptr(pIntAttrCount,   "%d",   buffer1));
  fprintf(fp, "  IntAttrs      : %s\n", strptr(ppIntAttrs,      "0x%p",   buffer1));
  fprintf(fp, "  CharAttrLength: %s\n", strptr(pCharAttrLength, "%d",   buffer1));
  DumpHex(fp, *ppIntAttrs, *pIntAttrCount * sizeof(MQLONG)   );
  fprintf(fp, "  CharAttrs     : %s\n", strptr(ppCharAttrs,     "0x%p",   buffer1));
  DumpHex(fp, *ppCharAttrs, *pCharAttrLength                 );

  return;
}

/*********************************************************************/
/*                                                                   */
/* After MQSET Entrypoint                                            */
/*                                                                   */
/*********************************************************************/

MQ_SET_EXIT SetAfter;

void MQENTRY SetAfter    ( PMQAXP    pExitParms
                         , PMQAXC    pExitContext
                         , PMQHCONN  pHconn
                         , PMQHOBJ   pHobj
                         , PMQLONG   pSelectorCount
                         , PPMQLONG  ppSelectors
                         , PMQLONG   pIntAttrCount
                         , PPMQLONG  ppIntAttrs
                         , PMQLONG   pCharAttrLength
                         , PPMQCHAR  ppCharAttrs
                         , PMQLONG   pCompCode
                         , PMQLONG   pReason
                         )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, TITLE_FORMAT, "AFTER", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "  CompCode      : %s\n", strptr(pCompCode, "%d", buffer1));
  fprintf(fp, "  Reason        : %s\n", strptr(pReason,   "%d", buffer1));
  fprintf(fp, "--------------------------------------------------------------------------\n");

  return;
}

/*********************************************************************/
/*                                                                   */
/* TERM                                                              */
/*                                                                   */
/*********************************************************************/

MQ_TERM_EXIT Terminate;

void MQENTRY Terminate ( PMQAXP   pExitParms
                       , PMQAXC   pExitContext
                       , PMQLONG  pCompCode
                       , PMQLONG  pReason
                       )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = *ppExitUserArea;
  FILE            * fp             = pExitUserArea->fp;
  char              buffer1[50]    = "";
  char              buffer2[50]    = "";

  myGetAbsoluteTime( buffer1, sizeof(buffer1) );
  myGetRelativeTime( buffer2, pExitUserArea );

  fprintf(fp, "MQAPI Trace\n");
  fprintf(fp, TITLE_FORMAT, "END", buffer1, buffer2);

  if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_ALWAYS)
    DumpParms( fp, pExitParms );

  if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_ALWAYS)
    DumpContext( fp, pExitContext );

  fprintf(fp, "--------------------------------------------------------------------------\n");

  fclose(fp);
  free( pExitUserArea );

  return;
}

/*********************************************************************/
/*                                                                   */
/* Initialisation function                                           */
/*                                                                   */
/*********************************************************************/

MQ_INIT_EXIT EntryPoint;

void MQENTRY EntryPoint ( PMQAXP   pExitParms
                        , PMQAXC   pExitContext
                        , PMQLONG  pCompCode
                        , PMQLONG  pReason
                        )
{
  MQLONG            rc             = MQRC_NONE;
  MYEXITUSERAREA ** ppExitUserArea = (void*) &pExitParms->ExitUserArea;
  MYEXITUSERAREA  * pExitUserArea  = NULL;
  FILE            * fp             = NULL;
  char            * env            = NULL;
  char              logfile[1024]  = "";
  MQPID             pid            = 0;
  MQPID             tid            = 0;

  /*******************************************************************/
  /* Malloc storage for the ExitUserArea                             */
  /*******************************************************************/

  if (rc == MQRC_NONE)
  {
    pExitUserArea = calloc( 1, sizeof(MYEXITUSERAREA) );

    if (pExitUserArea)
      *ppExitUserArea = pExitUserArea;
    else
    {
      rc = MQRC_API_EXIT_ERROR;
      pExitParms->ExitResponse = MQXCC_FAILED;
    }
  }

  /*******************************************************************/
  /* Initialise the Windows Performance counter                      */
  /*******************************************************************/

#if (MQAT_DEFAULT == MQAT_WINDOWS_NT)
  if (rc == MQRC_NONE)  InitPerformanceCounter( pExitUserArea );
#endif

  /*******************************************************************/
  /* Get the options                                                 */
  /*******************************************************************/

  if (rc == MQRC_NONE)
  {
    env = getenv( "MQAPI_TRACE_OPTIONS" );

    if (env)
      pExitUserArea->Options = atoi( env );

    else if (myBlankCheck( pExitParms->ExitData, MQ_EXIT_DATA_LENGTH ))
      pExitUserArea->Options = OPTIONS_DEFAULT;

    else
      pExitUserArea->Options = atoi(pExitParms->ExitData);
  }

  /*******************************************************************/
  /* Write to the logfile                                            */
  /*******************************************************************/

  if (rc == MQRC_NONE)
  {
    env = getenv( "MQAPI_TRACE_LOGFILE" );

    if (env)
    {
      /***************************************************************/
      /* Open the log file                                           */
      /***************************************************************/

      sprintf( logfile, "%s.%d.%d.log", env, pExitContext->ProcessId, pExitContext->ThreadId );

      fp = fopen( logfile, "w" );

      if (fp == NULL)
      {
        rc = MQRC_API_EXIT_ERROR;
        pExitParms->ExitResponse = MQXCC_FAILED;
      }
      else
      {
        char buffer1[50] = "";
        char buffer2[50] = "";

        myGetAbsoluteTime( buffer1, sizeof(buffer1) );
        myGetRelativeTime( buffer2, pExitUserArea );

        pExitUserArea->fp = fp;
        fprintf(fp, "MQAPI Trace\n");
        fprintf(fp, TITLE_FORMAT, "START", buffer1, buffer2);

        if (pExitUserArea->Options & OPTIONS_DUMP_PARMS_AT_START)
          DumpParms( fp, pExitParms );

        if (pExitUserArea->Options & OPTIONS_DUMP_CONTEXT_AT_START)
          DumpContext( fp, pExitContext );

        fprintf(fp, "--------------------------------------------------------------------------\n");
      }

      /***************************************************************/
      /* Register the MQBACK entrypoints                             */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_BACK
              , (PMQFUNC) BackBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_BACK
              , (PMQFUNC) BackAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQBEGIN entrypoints                            */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_BEGIN
              , (PMQFUNC) BeginBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_BEGIN
              , (PMQFUNC) BeginAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQCLOSE entrypoints                            */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_CLOSE
              , (PMQFUNC) CloseBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_CLOSE
              , (PMQFUNC) CloseAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQCMIT entrypoints                             */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_CMIT
              , (PMQFUNC) CmitBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_CMIT
              , (PMQFUNC) CmitAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQCONN entrypoints                             */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_CONN
              , (PMQFUNC) ConnBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_CONN
              , (PMQFUNC) ConnAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQCONNX entrypoints                            */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_CONNX
              , (PMQFUNC) ConnxBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_CONNX
              , (PMQFUNC) ConnxAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQDISC entrypoints                             */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_DISC
              , (PMQFUNC) DiscBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_DISC
              , (PMQFUNC) DiscAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQGET entrypoints                              */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_GET
              , (PMQFUNC) GetBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if ((rc == MQRC_NONE) && (pExitUserArea->Options & OPTIONS_DUMP_DATACONV))
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_DATA_CONV_ON_GET
              , (PMQFUNC) GetBeforeDataConv
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_GET
              , (PMQFUNC) GetAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQINQ entrypoints                              */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_INQ
              , (PMQFUNC) InqBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_INQ
              , (PMQFUNC) InqAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQOPEN entrypoints                             */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_OPEN
              , (PMQFUNC) OpenBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_OPEN
              , (PMQFUNC) OpenAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQPUT entrypoints                              */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_PUT
              , (PMQFUNC) PutBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_PUT
              , (PMQFUNC) PutAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQPUT1 entrypoints                             */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_PUT1
              , (PMQFUNC) Put1Before
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_PUT1
              , (PMQFUNC) Put1After
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the MQSET entrypoints                              */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_BEFORE
              , MQXF_SET
              , (PMQFUNC) SetBefore
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_AFTER
              , MQXF_SET
              , (PMQFUNC) SetAfter
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }

      /***************************************************************/
      /* Register the termination entrypoint                         */
      /***************************************************************/

      if (rc == MQRC_NONE)
      {
        MQXEP ( pExitParms->Hconfig
              , MQXR_CONNECTION
              , MQXF_TERM
              , (PMQFUNC) Terminate
              , NULL
              , pCompCode
              , pReason
              );

        if (*pReason != MQRC_NONE)
        {
          rc = *pReason;
          pExitParms->ExitResponse = MQXCC_FAILED;
        }
      }
    }
  }

  return;
}


