/*********************************************************************/
/*                                                                   */
/* Module Name: AMQODQUA.C                                           */
/*                                                                   */
/* Description: Common source for the MQSeries Dead letter handler.  */
/* <START_COPYRIGHT>                                                 */
/*  Statement:     Licensed Materials - Property of IBM              */
/*                                                                   */
/*                 04L1773, 5765-B73                                 */            
/*                 04L1802, 5639-B42                                 */            
/*                 04L1788, 5765-B74                                 */            
/*                 04L1816, 5765-B75                                 */            
/*                 04L1830, 5639-B43                                 */            
/*                 (C) Copyright IBM Corp. 1994, 1999                */
/* <END_COPYRIGHT>                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* This file is the common source file which contains all of the     */
/* functions which are private to the sample version of the dead     */
/* letter handler.                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <cmqc.h>
#include <amqodqha.h>


static int indent;            /* Trace indentation level            */
static FILE *MsgFile ;        /* Console messages                   */
static int odq_trace ;        /* Trace on/off indicator             */
static int odq_msg ;          /* Console messages enabled indicator */
static fpos_t *MsgIndex;      /* Console message file index         */
#define ODQ_FUNCTION_NAMES
struct {
     char name[40];
  } odq_function[] = {
#include <amqodtha.h>
};

#undef FUNCTION
#define FUNCTION odqtodqPreInitialize
/*********************************************************************/
/*                                                                   */
/* Function: odqPreInitialize                                        */
/*                                                                   */
/* Description: Enable basic MQ services                             */
/*                                                                   */
/* Intended Function: This function initializes basic MQSeries       */
/*                    services such as trace/dump etc.               */
/*                                                                   */
/* Input Parameters: none                                            */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
odqResp odqPreInitialize(void)
{
 char * trace ;
 char   Buffer[256] ;
 int    MsgId,matches ;
 fpos_t MsgPos ;
 odqResp rc = odq_Ok ;

/*--------------------------------------------------------------------*/
/* Use an environment variable to detect if tracing is to be active.  */
/*                                                                    */
/*--------------------------------------------------------------------*/
 trace = getenv( "ODQ_TRACE" );
 if( (NULL != trace) &&
     ( (strncmp(trace,"YES", strlen(trace)) == 0) |
       (strncmp(trace,"yes", strlen(trace)) == 0)  ) )
 {
   odq_trace = 1 ;
 }
 else
 {
   odq_trace = 0 ;
 }

/*--------------------------------------------------------------------*/
/* Now that tracing is initialized we can trace entry to this         */
/* function.                                                          */
/*--------------------------------------------------------------------*/
 odq_fnc_entry( FUNCTION );

/*--------------------------------------------------------------------*/
/* Open the message catalog.                                          */
/*--------------------------------------------------------------------*/
 MsgFile = fopen( getenv("ODQ_MSG"), "r" ) ;
 if ( NULL == MsgFile )
 {
   fprintf( stderr,
            "odq-000  Unable to open message catalog\n"
            "check environment variable \"ODQ_MSG\" is set to the\n"
            "path name of the message file\n") ;
   rc = odq_UnexpectedInitErr ;
 }
 else
 {
/*--------------------------------------------------------------------*/
/* Obtain storage for the message index.                              */
/*--------------------------------------------------------------------*/
   rc = odqGetMem ( sizeof( fpos_t )*odq_high_error,
                    "MessageIndex",
                    (void **)&MsgIndex ) ;
   if( odq_Error > rc )
   {
     memset( MsgIndex, 0, sizeof( fpos_t )*odq_high_error);
     /*---------------------------------------------------------------*/
     /* Browse the message file building an index as we go.           */
     /*---------------------------------------------------------------*/
     while( 0 == fgetpos( MsgFile, &MsgPos ) &&
            NULL != fgets( Buffer, sizeof(Buffer), MsgFile ))
     {
        matches = sscanf( Buffer, "odq-%d %*s", &MsgId  );
        /*------------------------------------------------------------*/
        /* Ignore any line that does not start with a message id.     */
        /*------------------------------------------------------------*/
        if( 0 < matches )
        {
          *(MsgIndex + MsgId) = MsgPos ;
        }
     }
     /*---------------------------------------------------------------*/
     /* Indicate that messages can be sent.                           */
     /*---------------------------------------------------------------*/
     odq_msg = 1 ;
   }
 }
 odq_fnc_retcode( FUNCTION, rc ) ;
 return rc;
}


#undef FUNCTION
#define FUNCTION odqtodqInitialize
/*********************************************************************/
/*                                                                   */
/* Function: odqInitialize                                           */
/*                                                                   */
/* Description: Perform second stage initialization                  */
/*                                                                   */
/* Intended Function: This function performs queue manager related   */
/*                    initialization which could not be performed    */
/*                    during PreInitialization as the queue  manager */
/*                    name was not known at that time.               */
/*                                                                   */
/* Input Parameters: none                                            */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
odqResp odqInitialize(void)
{
 odqResp rc = odq_Ok ;
 odq_fnc_entry( FUNCTION );
 /* currently no initialization required */
 odq_fnc_retcode( FUNCTION, rc ) ;
 return rc;
}


#ifdef AMQ_AS400
/*********************************************************************/
/*                                                                   */
/* Function: odqTerminate                                            */
/*                                                                   */
/* Description: Perform termination                                  */
/*                                                                   */
/* Intended Function: This function is a NULL operation for program  */
/*                    AMQSDLQ.                                       */
/*                                                                   */
/* Input Parameters: none                                            */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
odqResp odqTerminate(void)
{
  return odq_Ok;                                     /* AMQSDLQ only */
}
#endif




#undef FUNCTION
#define FUNCTION odqtodqFFDC
/*********************************************************************/
/*                                                                   */
/* Function: odqFFDC                                                 */
/*                                                                   */
/* Description: Perform first failure data capture                   */
/*                                                                   */
/* Intended Function: This function provides first failure data      */
/*                    capture facilities for the dead letter queue   */
/*                    handler.                                       */
/*                                                                   */
/* Input Parameters: function in which the error occured             */
/*                   error identifier                                */
/*                   numeric code 1                                  */
/*                   numeric code 2                                  */
/*                   variable number of triplets of the form         */
/*                       address                                     */
/*                       length                                      */
/*                       remark                                      */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
void odqFFDC( /*IN*/
                const int function,
                const odqResp  MessageId,
                const MQLONG SubCode1,
                const MQLONG SubCode2,
                ...
              /*OUT*/
            )
{


int        argCount=0;
va_list    listp;
void *     argp;
int        argl;
char       *argcomment;
char       *inserts[3];
time_t     now ;
struct tm  *ptime;
char       date[24] ;



odq_fnc_entry( FUNCTION );

/*--------------------------------------------------------------------*/
/* Build the date/time stamp for the message.                         */
/*--------------------------------------------------------------------*/
time (&now) ;
ptime = gmtime( &now ) ;
strftime( date, sizeof(date), "%d/%m/%y %H:%M:%S", ptime ) ;


/*--------------------------------------------------------------------*/
/* Convert the variable length plist into an array of inserts.        */
/*--------------------------------------------------------------------*/
va_start(listp, SubCode2);
for( argp = va_arg(listp, void *),
     argl = va_arg(listp, int ) ,
     argcomment = va_arg(listp, char * ) ;
     argp != NULL;
     argCount ++
   )
{
  inserts[argCount] = argp ;
  argp = va_arg(listp, void *);
  if ( NULL != argp )
  {
    argl = va_arg(listp, int ) ;
    argcomment = va_arg(listp, char * ) ;
  }
}
va_end(listp);

/*--------------------------------------------------------------------*/
/* If messages are initialized then write the appropriate message.    */
/*--------------------------------------------------------------------*/
if ( 0 != odq_msg )
{
  /*------------------------------------------------------------------*/
  /* Locate the message via the index.                                */
  /*------------------------------------------------------------------*/
  if( 0 != fsetpos( MsgFile, MsgIndex+MessageId ))
  {
   fprintf( stderr, "%s odq-000: Message odq-%d not found\n",
            date, MessageId );
  }
  else
  {
   /*-----------------------------------------------------------------*/
   /* Look for the strings &1, &2,&3,&4,&5 which need to be replaced  */
   /* by the relevant inserts.                                        */
   /* And write the message to stderr.                                */
   /*-----------------------------------------------------------------*/
   char c;
   fprintf( stderr,
            "%s ",
            date);

   for( c = fgetc(MsgFile)  ;
       c != '\n' ;
       c = fgetc( MsgFile) )
   {
     if ( '&' == c )
     {
       c = fgetc(MsgFile) ;
       switch( c )
       {
         case '1': fprintf(stderr,"%d", SubCode1 );
           break;
         case '2': fprintf(stderr,"%d", SubCode2 );
           break;
         case '3': fprintf(stderr,"%s", inserts[0] );
           break;
         case '4': fprintf(stderr,"%s", inserts[1] );
           break;
         case '5': fprintf(stderr,"%s", inserts[2] );
           break;
         default:  fprintf(stderr,"MESSAGE_TEMPLATE_ERROR");
           break;
       }
     }
     else
     {
       fputc( c, stderr ) ;
     }
   }
   fputc( '\n', stderr ) ;
  }
}

/*--------------------------------------------------------------------*/
/* If this is a severe error then dump the inserts to stderr in       */
/* both hex and text format.                                          */
/*--------------------------------------------------------------------*/
if( odq_Disaster <= MessageId)
{
  char *p;
  int   i ;
  va_start(listp, SubCode2);
  for( argp = va_arg(listp, void *),
       argl = va_arg(listp, int ) ,
       argcomment = va_arg(listp, char * ) ;
       argp != NULL;
     )
  {
    fputs(argcomment, stderr) ;
    fputc('\n',stderr);
    for( p = argp;
         p < (char *)argp + argl -16 ;
         p += 16 )
    {
        for( i  = 0 ;
             i < 16 ;
             i ++  )
        {
          fprintf( stderr, "%02x", *(p+i) ) ;
          if( 3 == i%4)
          {
           fputc( ' ',stderr ) ;
          }
        }
        fputc('<', stderr) ;
        for( i  = 0 ;
             i < 16 ;
             i ++  )
        {
          if( isalnum(p[i])| ispunct(p[i]) )
          {
            fputc( *(p+i) ,stderr) ;
          }
          else
          {
            fputc( '.',stderr);
          }
        }
        fprintf(stderr,">\n") ;
    }
    if( argl%16 )
    {
     for( i  = 0 ;
          i < 16 ;
          i ++  )
     {
       if( argl%16 >= i )
       {
         fprintf( stderr, "%02x", *(p+i) ) ;
       }
       else
       {
        fputc( ' ',stderr ) ;
       }
       if( 3 == i%4)
       {
        fputc( ' ',stderr ) ;
       }
     }
     fputc('<', stderr) ;
     for( i  = 0 ;
          i < 16 ;
          i ++  )
     {
       if (argl%16 >= i )
       {
         if( isalnum(p[i])| ispunct(p[i]) )
         {
           fputc( *(p+i),stderr );
         }
         else
         {
           fputc( '.',stderr);
         }
       }
     }
     fprintf(stderr, ">\n" ) ;
    }
    fprintf(stderr, "%s\n", argcomment ) ;
    argp = va_arg(listp, void *);
    if ( NULL != argp )
    {
      argl = va_arg(listp, int ) ;
      argcomment = va_arg(listp, char * ) ;
    }
  }
  va_end(listp);
}


 odq_fnc_retcode( FUNCTION, odq_Ok ) ;
}


#undef  FUNCTION
#define FUNCTION odqtodqGetMem
/*********************************************************************/
/*                                                                   */
/* Function: odqGetMem                                               */
/*                                                                   */
/* Description: obtain private storage                               */
/*                                                                   */
/* Intended Function: This function is called to obtain private      */
/*                    storage for use by the DLQ handler.            */
/*                                                                   */
/* Input Parameters: length                                          */
/*                   remark                                          */
/*                                                                   */
/* Output Parameters: address                                        */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
odqResp odqGetMem( /*IN*/
                     int length,
                     char * comment,
                   /*OUT*/
                     void **address
                 )

{
 odqResp rc=odq_Ok ;
/*-------------------------------------------------------------------*/
/* Obtain private storage for use by the dead letter queue handler.  */
/*                                                                   */
/* note: The 'comment' input field is not used by this routine but   */
/*       is provided to allow the callers to identify the            */
/*       corresponding odqFreMem.                                    */
/*-------------------------------------------------------------------*/
 odq_fnc_entry( FUNCTION ) ;
 *address = malloc(length) ;
 if( NULL == *address )
 {
   odqFFDC( /*IN*/
              FUNCTION,
              odq_NoStorage,
              0,
              0,
              comment,
              strlen(comment),
              "Remark",
              NULL
            /*OUT*/
          ) ;
   rc = odq_NoStorage ;
 }
 odq_fnc_retcode( FUNCTION, rc ) ;
 return rc;
}
#undef  FUNCTION
#define FUNCTION odqtodqFreeMem
/*********************************************************************/
/*                                                                   */
/* Function: odqFreeMem                                              */
/*                                                                   */
/* Description: release private storage                              */
/*                                                                   */
/* Intended Function: This function is called to obtain private      */
/*                    storage for use by the DLQ handler.            */
/*                                                                   */
/* Input Parameters: remark                                          */
/*                   address                                         */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
odqResp odqFreeMem( /*IN*/
                      char * comment,
                      void * address
                    /*OUT*/
                  )

{
 odqResp rc=odq_Ok ;
/*-------------------------------------------------------------------*/
/* Release private storage used by the dead letter queue handler.    */
/*                                                                   */
/* note: The 'comment' input field is not used by this routine but   */
/*       is provided to allow the callers to identify the            */
/*       corresponding odqGetMem.                                    */
/*-------------------------------------------------------------------*/
 odq_fnc_entry( FUNCTION ) ;
 free( address ) ;
 odq_fnc_retcode( FUNCTION, rc ) ;
 return rc;
}


#undef  FUNCTION
/*********************************************************************/
/*                                                                   */
/* Function: odq_fnc_entry                                           */
/*                                                                   */
/* Description: Trace entry to a function                            */
/*                                                                   */
/* Intended Function: This function makes a trace entry for entry    */
/*                    to a function.                                 */
/*                                                                   */
/* Input Parameters: function identifier                             */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
void odq_fnc_entry(/*IN*/
                      const odqTraceId function
                   /*OUT*/
                 )
{
  if( odq_trace )
  {
    if( indent )
    {
      fprintf( stderr, "%*s",  indent, "->");
    }
    fprintf(stderr,"%s\n", &(odq_function[function]) );
    indent += 2 ;
  }
}


/*********************************************************************/
/*                                                                   */
/* Function: odq_fnc_retcode                                         */
/*                                                                   */
/* Description: Trace exit from a function                           */
/*                                                                   */
/* Intended Function: This function makes a trace entry on exit      */
/*                    from a function.                               */
/*                                                                   */
/* Input Parameters: function identifier                             */
/*                   return code                                     */
/*                                                                   */
/* Output Parameters: none                                           */
/*                                                                   */
/* Returns:                                                          */
/*********************************************************************/
void odq_fnc_retcode(/*IN*/
                      const odqTraceId function,
                      const odqResp retcode
                     /*OUT*/
                    )
{
  if( odq_trace )
  {
    indent -= 2 ;
    if( indent )
    {
      fprintf( stderr, "%*s", indent,"<-");
    }
    fprintf(stderr,"%s rc=%d\n", &odq_function[function],retcode );
  }
}

#define FUNCTION odqtodqMQCONN
/*********************************************************************/
/*                                                                   */
/* Function: odqMQCONN                                               */
/*                                                                   */
/* Description: Connect to a queue manager.                          */
/*                                                                   */
/* Intended Function: This function connects to queue manager.       */
/*                                                                   */
/* Input Parameters: Queue manager name                              */
/*                                                                   */
/* Output Parameters: CompCode                                       */
/*                    Reason                                         */
/*                                                                   */
/* Returns:                                                          */
/*                                                                   */
/*********************************************************************/
void odqMQCONN(char * qm,
               MQLONG *CompCode,
               MQLONG *Reason)
{
 odqResp rc = odq_Ok ;
 odq_fnc_entry( FUNCTION );
 MQCONN(  qm,
          &odqGlobal.odqHConn,
          CompCode,
          Reason );
 odq_fnc_retcode( FUNCTION, rc ) ;
}


#undef FUNCTION
